<?php
/**
 * File for class StopForumSpam definition in the coreplus project
 *
 * @author Charlie Powell <charlie@evalagency.com>
 * @copyright Copyright (C) 2009-2017  Charlie Powell
 * @license GNU Affero General Public License v3 <http://www.gnu.org/licenses/agpl-3.0.txt>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/agpl-3.0.txt.
 */

namespace SecuritySuite;

use Core\Date\DateTime;
use Core\Date\Timezone;
use Core\Filestore\Contents\ContentZIP;
use Core\Filestore\Directory;
use Core\Filestore\Factory;
use Core\Filestore\File;
use Core\Session;
use Exception;
use SystemLogModel;


/**
 * Created by JetBrains PhpStorm.
 * User: powellc
 * Date: 12/15/12
 * Time: 1:49 PM
 * Helper class to work with the stopforumspam.com system.
 */
class StopForumSpam {
	const URL_DAILY = 'http://www.stopforumspam.com/downloads/listed_ip_1_all.zip';

	/**
	 * Method that can be called on the /cron hooks to download the newest version of sfs's blacklist.
	 *
	 * This is designed to grab the daily version, so /cron/daily is the best location.
	 *
	 * @return bool
	 */
	public static function HookDaily(){
		// LIVE
		$remotefile = StopForumSpam::URL_DAILY;
		// TESTING
		//$remotefile = ROOT_PDIR . 'components/security-suite/listed_ip_1_all.zip'

		return self::ImportList($remotefile);
	}

	public static function ImportList($filename){
		// Download the latest blacklist from sfs and import it into the system.
		// Each record will be in a format such as
		// "5.144.176.232","7","2012-12-11 00:08:10"
		// IP, number of submissions, date

		$remotefile = Factory::File($filename);

		// Does it exist?
		if(!$remotefile->exists()){
			echo 'Unable to read file ' . $remotefile->getFilename() . ', it does not appear to exist!';
			return false;
		}

		$contents = $remotefile->getContentsObject();

		// Make sure it's a zip file.  If it's not, then we'll have problems.
		if(!$contents instanceof \Core\Filestore\Contents\ContentZIP){
			echo 'File ' . $remotefile->getFilename() . ' does not appear to be a zip file, aborting extraction!';
			return false;
		}

		$dest = 'tmp/sfs-blacklist/';
		/** @var $extracted Directory */
		$extracted = $contents->extract($dest);
		/** @var $file File */
		$file = $extracted->get('listed_ip_1_all.txt');

		// Do this line by line instead of reading the entire contents into memory.
		$fh = fopen($file->getFilename(), 'r');
		if(!$fh){
			echo 'Unable to open file ' . $file->getFilename() . ' for reading, aborting extraction!';
			return false;
		}
		$recordscount = 0;
		$newcount     = 0;
		$updatedcount = 0;
		$skippedcount = 0;
		while(!feof($fh)){
			$line = fgetcsv($fh);
			// Don't know why.....
			if(!$line[0]) continue;

			++$recordscount;

			// If the record count is too low to even care about... just skip it.
			if($line[1] <= 20){
				++$skippedcount;
				continue;
			}

			$record = \IpBlacklistModel::Find(['ip_addr = ' . $line[0] . '/32'], 1);
			if(!$record) {
				$record = new \IpBlacklistModel();
				$record->ip_addr = $line[0] . '/32';
			}

			$record->expires = DateTime::FormatString($line[2], DateTime::FORMAT_EPOCH, Timezone::TIMEZONE_GMT) + (SECONDS_ONE_DAY * 2);
			$record->comment = 'Automatic block from StopForumSpam.com for ' . $line[1] . ' spam submissions';

			if($record->checkExists()) {
				++$updatedcount;
			}
			else {
				++$newcount;
			}

			$record->save(true);
		}
		fclose($fh);

		\IpBlacklistModel::CommitSaves();

		echo 'Processed ' . $recordscount . ' records from ' . $remotefile->getFilename() . ' successfully!' . "\n";
		echo 'New Records: ' . $newcount . "\n";
		echo 'Updated Records: ' . $updatedcount . "\n";
		echo 'Skipped Records: ' . $skippedcount . "\n";
		return true;
	}
}
